﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.Net;
using System.IO;
using System.Security.Cryptography;

namespace Development_II_OAuth
{
    public partial class _Default : System.Web.UI.Page
    {
        #region "Constants"

        // Consumer key and secret
        const string CONSUMER_KEY = "";
        const string CONSUMER_SECRET = "";

        // Client key and secret
        const string CLIENT_TOKEN = "";
        const string CLIENT_SECRET = "";

        // Other constants
        const string OAUTH_VERSION = "1.0";
        const string SIGNATURE_METHOD = "HMAC-SHA1";
        const string TWITTER_URL = "http://api.twitter.com/1/statuses/update.json";
        const string PARAMS = "oauth_consumer_key={0}&oauth_nonce={1}&oauth_signature_method={2}&oauth_timestamp={3}&oauth_token={4}&oauth_version={5}&status={6}";
        const string HEADER_FORMAT = "OAuth oauth_nonce=\"{0}\", oauth_signature_method=\"{1}\", oauth_timestamp=\"{2}\", oauth_consumer_key=\"{3}\", oauth_token=\"{4}\", oauth_signature=\"{5}\", oauth_version=\"{6}\"";

        #endregion


        #region "Page events"

        /// <summary>
        /// Page Load().
        /// </summary>        
        protected void Page_Load(object sender, EventArgs e)
        {
        }
        

        /// <summary>
        /// Button send OnClick event.
        /// </summary>
        protected void btnSend_Click(object sender, EventArgs e)
        {
            // Prepare time stamp and nonce
            var nonce = Convert.ToBase64String(new ASCIIEncoding().GetBytes(DateTime.Now.Ticks.ToString()));
            string timeStamp = GetTimeStamp();

            // Prepare params for signing
            var baseString = string.Format(PARAMS,
                                        CONSUMER_KEY,
                                        nonce,
                                        SIGNATURE_METHOD,
                                        timeStamp,
                                        CLIENT_TOKEN,
                                        OAUTH_VERSION,
                                        Uri.EscapeDataString(GetMessage())
                              );

            baseString = string.Concat("POST&", Uri.EscapeDataString(TWITTER_URL), "&", Uri.EscapeDataString(baseString));

            // Prepare key
            string compositeKey = string.Concat(Uri.EscapeDataString(CONSUMER_SECRET), "&", Uri.EscapeDataString(CLIENT_SECRET));

            // Create signature
            string signature;
            using (HMACSHA1 hasher = new HMACSHA1(ASCIIEncoding.ASCII.GetBytes(compositeKey)))
            {
                signature = Convert.ToBase64String(hasher.ComputeHash(ASCIIEncoding.ASCII.GetBytes(baseString)));
            }           

            // Prepare authentication header
            var authHeader = string.Format(HEADER_FORMAT,
                                    Uri.EscapeDataString(nonce),
                                    Uri.EscapeDataString(SIGNATURE_METHOD),
                                    Uri.EscapeDataString(timeStamp),
                                    Uri.EscapeDataString(CONSUMER_KEY),
                                    Uri.EscapeDataString(CLIENT_TOKEN),
                                    Uri.EscapeDataString(signature),
                                    Uri.EscapeDataString(OAUTH_VERSION)
                            );

            // Get message body
            string body =  "status=" + Uri.EscapeDataString(GetMessage());

            // Post to twitter
            WebResponse response = PostToTwitter(TWITTER_URL, authHeader, body);

            lblSent.Text = "Your tweet has been sent!";
        }

        #endregion


        #region "Methods"

        /// <summary>
        /// Returns actual time stamp from 1.1.1970
        /// </summary>
        string GetTimeStamp()
        {
            TimeSpan timeSpan = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
            return Convert.ToInt64(timeSpan.TotalSeconds).ToString();
        }   
   

        /// <summary>
        /// Returns message which should be posted to Twitter.
        /// </summary>
        /// <returns></returns>
        string GetMessage()
        {
            string s = string.Empty;
            foreach(char c in txtTweet.Text.Reverse())
            {
                s += c;
            }

            return s;
        }


        /// <summary>
        /// Sends new tweet to twitter.
        /// </summary>
        /// <param name="apiUrl">API base URL.</param>
        /// <param name="authHeader">Authentication header.</param>
        /// <param name="body">Body.</param>
        WebResponse PostToTwitter(string apiUrl, string authHeader, string body)
        {
            ServicePointManager.Expect100Continue = false;

            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(apiUrl);
            request.Headers.Add("Authorization", authHeader);
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded";
            using (Stream stream = request.GetRequestStream())
            {
                byte[] content = ASCIIEncoding.ASCII.GetBytes(body);
                stream.Write(content, 0, content.Length);
            }
            
            return request.GetResponse();
        }

        #endregion
    }
}
